==> atmsupport-1.0.tar.gz: atmsupport-1.0/libunitcl/ie_bhli.c <==
/*
 * Copyright (c) 1996-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/libunitcl/ie_bhli.c,v 1.8 2004/01/02 15:15:54 hbb Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include "inctcl.h"

#include <netnatm/unimsg.h>
#include <netnatm/msg/unistruct.h>
#if 0
#include <netnatm/api/unisap.h>
#endif

#include "unitcl.h"
#include "unitcl_priv.h"

REGISTER_IE(UNI_IE_BHLI, bhli, 0)

/*
 * {bhli type [byte ...]}
 */
int
parse_bhli(Tcl_Interp *interp, int argc, const char **argv,
    union uni_ieall *ie)
{
	u_int n;

	if(argc < 1 || argc > 8) {
		unitcl_setres(interp, "bad # of args for bhli");
		return TCL_ERROR;
	}

	if(strcmp(argv[0], "iso") == 0)
		ie->bhli.type = UNI_BHLI_ISO;
	else if(strcmp(argv[0], "user") == 0)
		ie->bhli.type = UNI_BHLI_USER;
	else if(strcmp(argv[0], "vendor") == 0)
		ie->bhli.type = UNI_BHLI_VENDOR;
	else {
		unitcl_setres(interp, "bad bhli type");
		return TCL_ERROR;
	}

	argc--;
	argv++;

	while(argc--) {
		if(unitcl_parse_num(interp, *argv++, &n) != TCL_OK)
			return TCL_ERROR;
		ie->bhli.info[ie->bhli.len++] = n;
	}

	IE_ADDPRESENT(ie->bhli);

	return TCL_OK;
}

FMT_IE_START(UNI_IE_BHLI, bhli)
{
	char buf[100];
	u_int i;

	if(ie->type == UNI_BHLI_ISO)
		Tcl_DStringAppendElement(str, "iso");
	else if(ie->type == UNI_BHLI_USER)
		Tcl_DStringAppendElement(str, "user");
	else if(ie->type == UNI_BHLI_VENDOR)
		Tcl_DStringAppendElement(str, "vendor");
	else {
		unitcl_setres(interp, "bad bhli type to format");
		return TCL_ERROR;
	}

	for(i = 0; i < ie->len; i++) {
		sprintf(buf, "0x%02x", ie->info[i]);
		Tcl_DStringAppendElement(str, buf);
	}
}
FMT_IE_END()

#if 0
/*
 * {bhli <tag> <type> [info ...]}
 */
int
parse_bhli_sve(Tcl_Interp *interp, const char *arg, struct unisve_bhli *sve)
{
	const char **argv, **av;
	char *end;
	int argc;
	int err = TCL_ERROR;
	u_long proto;

	if(Tcl_SplitList(interp, arg, &argc, &argv) != TCL_OK)
		return TCL_ERROR;
	av = argv;

	if(argc < 2) {
		unitcl_setres(interp, "bhli SVE: {bhli <tag> ...}");
		goto out;
	}
	if(strcmp(argv[0], "bhli") != 0) {
		unitcl_setres(interp, "SVE type not 'bhli': %s", argv[0]);
		goto out;
	}
	if(parse_svetag(interp, argv[1], &sve->tag) != TCL_OK)
		goto out;
	argc -= 2;
	argv += 2;

	if(sve->tag == UNISVE_PRESENT) {
		if(argc < 2) {
			unitcl_setres(interp, "bhli SVE needs type and info");
			goto out;
		}
		if(strcmp(argv[0], "vendor") == 0)
			sve->type = UNI_BHLI_VENDOR;
		else if(strcmp(argv[0], "user") == 0)
			sve->type = UNI_BHLI_USER;
		else if(strcmp(argv[0], "iso") == 0)
			sve->type = UNI_BHLI_ISO;
		else {
			unitcl_setres(interp, "bad bhli type '%s'", argv[0]);
			goto out;
		}
		argc--;
		argv++;

		if(argc > 8) {
			unitcl_setres(interp, "too much bhli info");
			goto out;
		}
		sve->len = 0;
		while(argc--) {
			proto = strtoul(argv[0], &end, 0);
			if(*end != '\0') {
				unitcl_setres(interp, "bad BHLI info '%s'",
				    argv[0]);
				goto out;
			}
			if(proto > 0xff) {
				unitcl_setres(interp, "BHLI info to large '%s'",
				    argv[0]);
				goto out;
			}
			sve->info[sve->len++] = (u_int8_t)proto;
			argv++;
		}
	}

	if(argc != 0) {
		unitcl_setres(interp, "excess args to bhli SVE");
		goto out;
	}

	err = TCL_OK;
  out:
	free(av);
	return err;
}
#endif

==> atmsupport-1.4.tar.gz: atmsupport-1.4/libunitcl/ie_bhli.c <==
/*
 * Copyright (c) 1996-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * $Begemot: atmsupport/libunitcl/ie_bhli.c,v 1.10 2004/05/12 11:50:04 novo Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include "inctcl.h"

#include <netnatm/unimsg.h>
#include <netnatm/msg/unistruct.h>
#include <netnatm/api/unisap.h>

#include "unitcl.h"
#include "unitcl_priv.h"

REGISTER_IE(UNI_IE_BHLI, bhli, 0)

/*
 * {bhli type [byte ...]}
 */
int
parse_bhli(Tcl_Interp *interp, int argc, const char **argv,
    union uni_ieall *ie)
{
	u_int n;

	if(argc < 1 || argc > 8) {
		unitcl_setres(interp, "bad # of args for bhli");
		return TCL_ERROR;
	}

	if(strcmp(argv[0], "iso") == 0)
		ie->bhli.type = UNI_BHLI_ISO;
	else if(strcmp(argv[0], "user") == 0)
		ie->bhli.type = UNI_BHLI_USER;
	else if(strcmp(argv[0], "vendor") == 0)
		ie->bhli.type = UNI_BHLI_VENDOR;
	else {
		unitcl_setres(interp, "bad bhli type");
		return TCL_ERROR;
	}

	argc--;
	argv++;

	while(argc--) {
		if(unitcl_parse_num(interp, *argv++, &n) != TCL_OK)
			return TCL_ERROR;
		ie->bhli.info[ie->bhli.len++] = n;
	}

	IE_ADDPRESENT(ie->bhli);

	return TCL_OK;
}

FMT_IE_START(UNI_IE_BHLI, bhli)
{
	char buf[100];
	u_int i;

	if(ie->type == UNI_BHLI_ISO)
		Tcl_DStringAppendElement(str, "iso");
	else if(ie->type == UNI_BHLI_USER)
		Tcl_DStringAppendElement(str, "user");
	else if(ie->type == UNI_BHLI_VENDOR)
		Tcl_DStringAppendElement(str, "vendor");
	else {
		unitcl_setres(interp, "bad bhli type to format");
		return TCL_ERROR;
	}

	for(i = 0; i < ie->len; i++) {
		sprintf(buf, "0x%02x", ie->info[i]);
		Tcl_DStringAppendElement(str, buf);
	}
}
FMT_IE_END()

/*
 * {bhli <tag> <type> [info ...]}
 */
int
parse_bhli_sve(Tcl_Interp *interp, const char *arg, struct unisve_bhli *sve)
{
	const char **argv, **av;
	char *end;
	int argc;
	u_long proto;

	if (Tcl_SplitList(interp, arg, &argc, &argv) != TCL_OK)
		return (TCL_ERROR);
	av = argv;

	if (argc < 2) {
		Tcl_Free((char *)av);
		return (unitcl_setres(interp, "bhli SVE: {bhli <tag> ...}"));
	}
	if (strcmp(argv[0], "bhli") != 0) {
		Tcl_Free((char *)av);
		return (unitcl_setres(interp,
		    "SVE type not 'bhli': %s", argv[0]));
	}
	if (parse_svetag(interp, argv[1], &sve->tag) != TCL_OK) {
		Tcl_Free((char *)av);
		return (TCL_ERROR);
	}
	argc -= 2;
	argv += 2;

	if (sve->tag == UNISVE_PRESENT) {
		if (argc < 2) {
			Tcl_Free((char *)av);
			return (unitcl_setres(interp,
			    "bhli SVE needs type and info"));
		}
		if (strcmp(argv[0], "vendor") == 0)
			sve->type = UNI_BHLI_VENDOR;
		else if (strcmp(argv[0], "user") == 0)
			sve->type = UNI_BHLI_USER;
		else if (strcmp(argv[0], "iso") == 0)
			sve->type = UNI_BHLI_ISO;
		else {
			Tcl_Free((char *)av);
			return (unitcl_setres(interp,
			    "bad bhli type '%s'", argv[0]));
		}
		argc--;
		argv++;

		if (argc > 8) {
			Tcl_Free((char *)av);
			return (unitcl_setres(interp, "too much bhli info"));
		}
		sve->len = 0;
		while (argc--) {
			proto = strtoul(argv[0], &end, 0);
			if (*end != '\0') {
				Tcl_Free((char *)av);
				return (unitcl_setres(interp,
				    "bad BHLI info '%s'", argv[0]));
			}
			if (proto > 0xff) {
				Tcl_Free((char *)av);
				return (unitcl_setres(interp,
				    "BHLI info to large '%s'", argv[0]));
			}
			sve->info[sve->len++] = (uint8_t)proto;
			argv++;
		}
	}

	if (argc != 0) {
		Tcl_Free((char *)av);
		return (unitcl_setres(interp, "excess args to bhli SVE"));
	}

	Tcl_Free((char *)av);
	return (TCL_OK);
}

int
fmt_bhli_sve(Tcl_Interp *interp, Tcl_DString *str,
    const struct unisve_bhli *sve)
{
	char buf[100];
	u_int i;

	Tcl_DStringStartSublist(str);

	Tcl_DStringAppendElement(str, "bhli");
	if (fmt_svetag(interp, str, sve->tag) != TCL_OK)
		return (TCL_ERROR);

	if (sve->tag == UNISVE_PRESENT) {

		if (sve->type == UNI_BHLI_VENDOR)
			Tcl_DStringAppendElement(str, "vendor");
		else if (sve->type == UNI_BHLI_USER)
			Tcl_DStringAppendElement(str, "user");
		else if (sve->type == UNI_BHLI_ISO)
			Tcl_DStringAppendElement(str, "iso");
		else
			return (unitcl_setres(interp,
			    "bad bhli type %u", sve->type));

		for (i = 0; i < sve->len; i++) {
			sprintf(buf, "%u", sve->info[i]);
			Tcl_DStringAppendElement(str, buf);
		}
	}

	Tcl_DStringEndSublist(str);
	return (TCL_OK);
}
